//DEFINEs
#define WEBSERVER_ESP_WIFI_SSID      "default"
#define WEBSERVER_ESP_WIFI_PASS      "default123"
#define WEBSERVER_ESP_MAXIMUM_RETRY      10
#define WEBSERVER_ESP_STATION_PASSWORD  "test_123"
#define WEBSERVER_ESP_STATION_NAME      "Schaltsteckdose"
#define GPIO_OUTPUT_IO_0    15
#define GPIO_OUTPUT_IO_1    16
#define GPIO_INPUT_IO_0     12
#define GPIO_INPUT_IO_1     14
#define GPIO_INPUT_PIN_SEL  ((1ULL<<GPIO_INPUT_IO_0) | (1ULL<<GPIO_INPUT_IO_1))
#define GPIO_OUTPUT_PIN_SEL  ((1ULL<<GPIO_OUTPUT_IO_0) | (1ULL<<GPIO_OUTPUT_IO_1))
#define WIFI_CONNECTED_BIT BIT0
#define WIFI_FAIL_BIT      BIT1
#define PORT_OFF            0
#define PORT_ON             1
#define CONFIG_MODE         2
#define CONFIG_SAVE         3
#define CONFIG_FAILED       4

//Global variables

//TAG
static const char *TAG="webserver";

//our website
char user_str[4096];
time_t now;
struct tm timeinfo;
char strftime_buf[64];
    
/* FreeRTOS event group to signal when we are connected*/
static EventGroupHandle_t s_wifi_event_group;

static int s_retry_num = 0;
static httpd_handle_t server = NULL;

// status = Port status (0=off, 1=on), wifi state = wifi status (0=not connected, 1=connected), button pressed = indicator for button pressed
// delay count = delay for button debounce, next_switch_delay = delay between port switches, startup_in_progress = indicator for boot process
// cfg_err = used to store config mistakes temporary in a bit mask
int status = 0, wifi_state = 0, button_pressed = 0, delay_count = 0, next_switch_delay = 0, startup_in_progress = 0, cfg_err = 0;

//Functions

//http.c 
void generate_response(int gen_page_id); 

//init.c
void init_spiffs(void);
void init_ports(void);
static void init_sntp(void);
void wifi_init_sta(void);

//interrupt.c
static void IRAM_ATTR gpio_isr_handler(void *arg);

//server.c
esp_err_t cfgsav_get_handler(httpd_req_t *req);
esp_err_t config_get_handler(httpd_req_t *req);
esp_err_t switch_get_handler(httpd_req_t *req);
esp_err_t root_get_handler(httpd_req_t *req);
httpd_handle_t start_webserver(void);
void stop_webserver(httpd_handle_t server);

//util.c
void config_read(void);
void config_write(int fstatus);
static void connect_handler(void* arg, esp_event_base_t event_base, int32_t event_id, void* event_data);
static void disconnect_handler(void* arg, esp_event_base_t event_base,int32_t event_id, void* event_data);
static void event_handler(void* arg, esp_event_base_t event_base,int32_t event_id, void* event_data);
char from_hex(char ch);
int isIp_v4( char* ip);
static void obtain_time(void);
void port_switch(int port_status);
char *url_decode(char *str);
int valid_digit(char* ip_str);

//Structs 

//URI handler structs
httpd_uri_t cfgsav_uri = {
    .uri       = "/cfgsav",
    .method    = HTTP_GET,
    .handler   = cfgsav_get_handler,
    /* Let's pass response string in user
     * context to demonstrate it's usage */
    .user_ctx  = NULL
};

httpd_uri_t config_uri = {
    .uri       = "/config",
    .method    = HTTP_GET,
    .handler   = config_get_handler,
    /* Let's pass response string in user
     * context to demonstrate it's usage */
    .user_ctx  = NULL
};

httpd_uri_t switch_uri = {
    .uri       = "/switch",
    .method    = HTTP_GET,
    .handler   = switch_get_handler,
    /* Let's pass response string in user
     * context to demonstrate it's usage */
    .user_ctx  = NULL
};

httpd_uri_t root_uri = {
    .uri       = "/",
    .method    = HTTP_GET,
    .handler   = root_get_handler,
    /* Let's pass response string in user
     * context to demonstrate it's usage */
    .user_ctx  = NULL
};

//configuration
typedef struct
{
    char ssid[34];      //Wlan SSID 
    char password[66];  //Wlan password
    char name[34];      //Station name
    char ip[18];        //IP address
    char gw[18];        //Default gateway
    char netmask[18];   //Subnet mask
    char use_dhcp[5];      //use DHCP
    char station_password[66];  //Station password
    char use_authorization[5];  //Authorize switch actions via password
    
}
config_struct;

config_struct config;
