/* read config */
void config_read (void)
{
    ESP_LOGI(TAG, "Reading file");
    FILE* f = fopen("/spiffs/status.txt", "r");
    if (f == NULL) 
        {
        ESP_LOGE(TAG, "Failed to open file for reading");
        return;
        }
    char line[16];
    fgets(line, sizeof(line), f);
    fgets(config.ssid, sizeof(config.ssid), f);
    fgets(config.password, sizeof(config.password), f);
    fgets(config.name, sizeof(config.name), f);
    fgets(config.ip, sizeof(config.ip), f);
    fgets(config.gw, sizeof(config.gw), f);
    fgets(config.netmask, sizeof(config.netmask), f);
    fgets(config.use_dhcp, sizeof(config.use_dhcp), f);
    fgets(config.station_password, sizeof(config.station_password), f);
    fgets(config.use_authorization, sizeof(config.use_authorization), f);
    fclose(f);
    // strip newline
    char* pos = strchr(line, '\n');
    if (pos) 
        *pos = '\0';
    char *pos1 = strchr(config.ssid, '\n');
    if (pos1) 
        *pos1 = '\0';
    char *pos2 = strchr(config.password, '\n');
    if (pos2) 
        *pos2 = '\0';
    char *pos3 = strchr(config.name, '\n');
    if (pos3) 
        *pos3 = '\0';
    char *pos4 = strchr(config.ip, '\n');
    if (pos4) 
        *pos4 = '\0';
    char *pos5 = strchr(config.gw, '\n');
    if (pos5) 
        *pos5 = '\0';
    char *pos6 = strchr(config.netmask, '\n');
    if (pos6) 
        *pos6 = '\0';
    char *pos7 = strchr(config.use_dhcp, '\n');
    if (pos7) 
        *pos7 = '\0';
    char *pos8 = strchr(config.station_password, '\n');
    if (pos8) 
        *pos8 = '\0';
    char *pos9 = strchr(config.use_authorization, '\n');
    if (pos9) 
        *pos9 = '\0';
    status = atoi (line);
    if (*config.ssid)
        ESP_LOGI(TAG, "Read from file: station name:%s,status:%i,SSID:%s,ip:%s,gw:%s,netmask:%s, use_dhcp:%s, use_authorization:%s", config.name,status,config.ssid,config.ip,config.gw,config.netmask,config.use_dhcp,config.use_authorization);
    else
        {
        status = PORT_OFF;
        strcpy(config.name, WEBSERVER_ESP_STATION_NAME);
        strcpy(config.ssid, WEBSERVER_ESP_WIFI_SSID);
        strcpy(config.password, WEBSERVER_ESP_WIFI_PASS);
        strcpy(config.station_password, WEBSERVER_ESP_STATION_PASSWORD);
        strcpy(config.use_dhcp,"yes");
        ESP_LOGI(TAG, "Config read failed, trying to connect to SSID %s with password %s.",config.ssid, config.password);
        }
}

/* write config */
void config_write (int fstatus)
{
    ESP_LOGI(TAG, "Opening file");
    FILE* f = fopen("/spiffs/status.txt", "w");
    if (f == NULL) 
        {
        ESP_LOGE(TAG, "Failed to open file for writing");
        return;
        }
    fprintf(f, "%i\n",fstatus);
    fprintf(f, "%s\n",(char *)config.ssid);
    fprintf(f, "%s\n",(char *)config.password);
    fprintf(f, "%s\n",(char *)config.name);
    fprintf(f, "%s\n",(char *)config.ip);
    fprintf(f, "%s\n",(char *)config.gw);
    fprintf(f, "%s\n",(char *)config.netmask);
    fprintf(f, "%s\n",(char *)config.use_dhcp);
    fprintf(f, "%s\n",(char *)config.station_password);
    fprintf(f, "%s\n",(char *)config.use_authorization);
    fclose(f);
    ESP_LOGI(TAG, "File written");
}

static void connect_handler(void* arg, esp_event_base_t event_base, int32_t event_id, void* event_data)
{
    httpd_handle_t* server = (httpd_handle_t*) arg;
    wifi_state = 1;
    if (*server == NULL) 
        {
        ESP_LOGI(TAG, "Starting webserver");
        *server = start_webserver();
        }
}

/*handlers for connect/disconnect */
static void disconnect_handler(void* arg, esp_event_base_t event_base, int32_t event_id, void* event_data)
{
    s_wifi_event_group = xEventGroupCreate();
    httpd_handle_t* server = (httpd_handle_t*) arg;
    wifi_state = 0;
    if (*server) 
        {
        ESP_LOGI(TAG, "Stopping webserver");
        stop_webserver(*server);
        *server = NULL;
        //webserver will be restarted by handler connect_handler again after wifi reconnect.
        }
  
}

/* event handler */
static void event_handler(void* arg, esp_event_base_t event_base,int32_t event_id, void* event_data)
{
    if (event_base == WIFI_EVENT && event_id == WIFI_EVENT_STA_START) 
        esp_wifi_connect();
    else if (event_base == WIFI_EVENT && event_id == WIFI_EVENT_STA_DISCONNECTED) 
        {
        if (s_retry_num < WEBSERVER_ESP_MAXIMUM_RETRY) 
            {
            esp_wifi_connect();
            s_retry_num++;
            ESP_LOGI(TAG, "retry to connect to the AP");
            } 
        else
            xEventGroupSetBits(s_wifi_event_group, WIFI_FAIL_BIT);
        ESP_LOGI(TAG,"connect to the AP fail");
        } 
    else if (event_base == IP_EVENT && event_id == IP_EVENT_STA_GOT_IP) 
        {
        ip_event_got_ip_t* event = (ip_event_got_ip_t*) event_data;
        ESP_LOGI(TAG, "got ip:%s",ip4addr_ntoa(&event->ip_info.ip));
        s_retry_num = 0;
        xEventGroupSetBits(s_wifi_event_group, WIFI_CONNECTED_BIT);
        }
}

/* Converts a hex character to its integer value */
char from_hex(char ch) 
{
    return isdigit(ch) ? ch - '0' : tolower(ch) - 'a' + 10;
}

int isIp_v4(char* ip)
{
    int num, dots = 0;
    char* ptr;

    if (ip == NULL)
        return 0;

    ptr = strtok(ip, ".");

    if (ptr == NULL)
        return 0;

    while (ptr) {

        /* after parsing string, it must contain only digits */
        if (!valid_digit(ptr))
            return 0;

        num = atoi(ptr);

        /* check for valid IP */
        if (num >= 0 && num <= 255) {
            /* parse remaining string */
            ptr = strtok(NULL, ".");
            if (ptr != NULL)
                ++dots;
        }
        else
            return 0;
    }

    /* valid IP string must contain 3 dots */
    if (dots != 3)
        return 0;
    return 1;
}

/* obtain time */
static void obtain_time(void)
{
    // wait for time to be set
    time_t now = 0;
    struct tm timeinfo = { 0 };
    int retry = 0;
    const int retry_count = 10;

    while (timeinfo.tm_year < (2016 - 1900) && ++retry < retry_count) 
        {
        ESP_LOGI(TAG, "Waiting for system time to be set... (%d/%d)", retry, retry_count);
        vTaskDelay(2000 / portTICK_PERIOD_MS);
        time(&now);
        localtime_r(&now, &timeinfo);
        }
} 

// toggle output port
void port_switch(int port_status)
{
    if (next_switch_delay > 0 || port_status == status)
        return;
    /*update time */
    time(&now);
    localtime_r(&now, &timeinfo);
    strftime(strftime_buf, sizeof(strftime_buf), "%c", &timeinfo);
    if (port_status)
        {
        strcat(strftime_buf," UTC / on!");
        ESP_LOGI(TAG, "Toggling PORT: on");
        }
    else
        {
        strcat(strftime_buf," UTC / off!");
        /*toggle Port */
        ESP_LOGI(TAG, "Toggling PORT: off");
        }
    next_switch_delay = 50; // wait a bit until next switch is allowed
    gpio_set_level(GPIO_OUTPUT_IO_0, port_status); 
    status = port_status;
    generate_response(port_status);
    config_write (port_status);
}
    
/* Returns a url-decoded version of str */
/* IMPORTANT: be sure to free() the returned string after use */
char *url_decode(char *str) 
    {
    char *pstr = str, *buf = malloc(strlen(str) + 1), *pbuf = buf;
    while (*pstr) 
        {
        if (*pstr == '%') 
            {
            if (pstr[1] && pstr[2]) 
                {
                *pbuf++ = from_hex(pstr[1]) << 4 | from_hex(pstr[2]);
                pstr += 2;
                }
            } 
        else if (*pstr == '+') 
            { 
            *pbuf++ = ' ';
            } 
        else 
            {
            *pbuf++ = *pstr;
            }
        pstr++;
        }
    *pbuf = '\0';
    return buf;
}

int valid_digit(char* ip_str)
{
    while (*ip_str) {
        if (*ip_str >= '0' && *ip_str <= '9')
            ++ip_str;
        else
            return 0;
    }
    return 1;
}
