/* HTTP Server 
includes Port switching on/off and logs time in UTC for each command.
*/

#include <sys/param.h>
#include <time.h>
#include "esp8266/gpio_struct.h"
#include "arpa/inet.h"

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/event_groups.h"
#include "esp_system.h"
#include "esp_log.h"
#include "esp_netif.h"
#include "esp_event.h"
#include "esp_wifi.h"
#include "nvs.h"
#include "nvs_flash.h"
#include "driver/gpio.h"
#include "esp_spiffs.h"

#include <esp_http_server.h>
#include "lwip/apps/sntp.h"
#include "files/header/webserver.h"
#include "files/http.c"
#include "files/init.c"
#include "files/interrupt.c"
#include "files/server.c"
#include "files/util.c"

/* main */

void app_main()
{
    //tell isr button is used for setting default config during startup
    startup_in_progress = 1;
 
    int wifi_reconnect_delay_count = 0;
    //init ports
    init_ports();
    gpio_set_level(GPIO_OUTPUT_IO_0, 0);
    //init file system 
    init_spiffs();
    //read config
    config_read();
    //init WIFI & Webserver
    ESP_ERROR_CHECK(nvs_flash_init());
    ESP_ERROR_CHECK(esp_netif_init());
    wifi_init_sta();
    if (status == PORT_ON)
        gpio_set_level(GPIO_OUTPUT_IO_0, PORT_ON);
    else
        gpio_set_level(GPIO_OUTPUT_IO_0, PORT_OFF);
    ESP_ERROR_CHECK(esp_event_handler_register(IP_EVENT, IP_EVENT_STA_GOT_IP, &connect_handler, &server));
    ESP_ERROR_CHECK(esp_event_handler_register(WIFI_EVENT, WIFI_EVENT_STA_DISCONNECTED, &disconnect_handler, &server));
    
    //init sntp
    if (wifi_state)
        {
        server = start_webserver();
        init_sntp();
        obtain_time();
        }
    setenv("TZ", "MESZ", 1);
    tzset();
    time(&now);
    localtime_r(&now, &timeinfo);
    strftime(strftime_buf, sizeof(strftime_buf), "%c", &timeinfo);
    ESP_LOGI(TAG, "The current date/time in Berlin is: %s", strftime_buf);
    if (status == PORT_ON)
        strcat(strftime_buf," UTC / on!");
    else
        strcat(strftime_buf," UTC / off!");
    
    //tell button ISR that button is now used to toggle port on/off
    startup_in_progress = 0;
    
    while (1)
        {
        vTaskDelay(10);  
        //minimum time frame between 2 port switches
        if (next_switch_delay > 0)
            next_switch_delay--;

        //debounce routine for button, the ISR only sets button_pressed to 1 on falling edge (if button is pressed)
        //wait until button is released (port input0 is high) and then decrease delay counter for debounce.
        //once its 0, toggle port status.
        if (delay_count > 0 && gpio_get_level(GPIO_INPUT_IO_0) != 0)
            delay_count--;
        if (button_pressed == 1 && delay_count == 0 && next_switch_delay == 0)
            {
            status = !status;
            config_write (status);
            ESP_LOGI(TAG, "Button was pressed, new status %i:", status);
            if (status == PORT_OFF)
                gpio_set_level(GPIO_OUTPUT_IO_0, PORT_OFF);
            else
                gpio_set_level(GPIO_OUTPUT_IO_0, PORT_ON);
            next_switch_delay = 50;  // wait a bit until next switch is allowed
            button_pressed = 0;
            }
        if (wifi_state == 0)
            {
            wifi_reconnect_delay_count++;
            if (wifi_reconnect_delay_count == 50)
                {
                wifi_reconnect_delay_count = 0;
                ESP_LOGI(TAG, "Try reconnect...");
                esp_wifi_connect();
                }
            }
        }
}
